/*
   ITU-T G.729A Speech Coder    ANSI-C Source Code
   Version 1.1    Last modified: September 1996

   Copyright (c) 1996,
   AT&T, France Telecom, NTT, Universite de Sherbrooke
   All rights reserved.
*/

/*-------------------------------------------------------------------*
 * Main program of the ITU-T G.729A  8 kbit/s encoder.               *
 *                                                                   *
 *    Usage : coder speech_file  synthethic_file                      *
 *-------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>

#include "typedef.h"
#include "basic_op.h"
#include "ld8a.h"

/*
 * Questa funzione raccoglie in 'payload' i soli 10 byte informativi di ogni frame.
 * Tali byte vengono presi dai 22 byte di 'prm', che, evidentemente, contiene 12 byte
 * ridondanti.
 * L'informazione contenuta in ogni elemento di 'prm'  allocata in:
 * 
 * parameter   position in prm[i]
 * L0+L1       1+7 lsb of prm[0]
 * L2+L3       5+5 lsb of prm[1]
 * P1            8 lsb of prm[2]
 * P0            1 lsb of prm[3]
 * C1           13 lsb of prm[4]
 * S1            4 lsb of prm[5]
 * F1+G1       3+4 lsb of prm[6]
 * P2            5 lsb of prm[7]
 * C2           13 lsb of prm[8]
 * S2            4 lsb of prm[9]
 * F2+G2       3+4 lsb of prm[10]
 * 
*/

void info_synt( unsigned char *payload, Word16 prm[PRM_SIZE])
{
	
 payload[0] = prm[0]&255;
 payload[1] = prm[1]&255;
 payload[2] = ((prm[1]&768)>>2)+(prm[2]&63);
 payload[3] = (prm[2]&192)+((prm[3]&1)<<5)+(prm[4]&31);         
 payload[4] = ((prm[4]&8160)>>5); 
 payload[5] = ((prm[5]&15)<<4)+(prm[6]&15);      
 payload[6]=  ((prm[6]&112)<<1)+(prm[7]&31);         
 payload[7] = prm[8]&255;      
 payload[8] = ((prm[8]&7936)>>5)+(prm[9]&7);
 payload[9] = ((prm[9]&8)<<4)+(prm[10]&127);    
 
}



int main(int argc, char *argv[] )
{
  FILE *f_speech;               /* File of speech data                   */
  FILE *f_synthetic;            /* Syntethic file for transmission       */
  unsigned char payload[10];
 
  extern Word16 *new_speech;     /* Pointer to new speech data            */

  Word16 prm[PRM_SIZE];          /* Analysis parameters.                  */

  Word16 frame;                  /* frame counter */
 
  printf("\n");
  printf("***********    ITU G.729A 8 KBIT/S SPEECH CODER    ***********\n");
  printf("\n");
  printf("------------------- Fixed point C simulation -----------------\n");
  printf("\n");
  printf("-------------------       Version 1.1        -----------------\n");
  printf("\n");


/*--------------------------------------------------------------------------*
 * Open speech file and result file (output synthetic file)                 *
 *--------------------------------------------------------------------------*/

  if ( argc != 3 )
    {
       printf("Usage : coder speech_file  synthetic_file\n");
       printf("\n");
       printf("Format for speech_file:\n");
       printf("  Speech is read from a binary file of 16 bits PCM data.\n");
       printf("\n");
       printf("Format syntethic_file:\n");
       printf("  Like bit packing proposed in 'draft-ietf-avt-profile'.\n");
       printf("\n");
       exit(1);
    }

  if ( (f_speech = fopen(argv[1], "rb")) == NULL) {
     printf("%s - Error opening file  %s !!\n", argv[0], argv[1]);
     exit(0);
  }
  printf(" Input speech file    :  %s\n", argv[1]);

  if ( (f_synthetic = fopen(argv[2], "wb")) == NULL) {
     printf("%s - Error opening file  %s !!\n", argv[0], argv[2]);
     exit(0);
  }
  printf(" Output synthetic file:  %s\n", argv[2]);

/*--------------------------------------------------------------------------*
 * Initialization of the coder.                                             *
 *--------------------------------------------------------------------------*/

  Init_Pre_Process();
  Init_Coder_ld8a();
  Set_zero(prm, PRM_SIZE);

  /* Loop for each "L_FRAME" speech data. */

  frame =0;
  while( fread(new_speech, sizeof(Word16), L_FRAME, f_speech) == L_FRAME)
  {
    printf("Frame =%d\r", frame++);

    Pre_Process(new_speech, L_FRAME);

    Coder_ld8a(prm);
   
    info_synt(payload, prm); // bit packing
    
    fwrite(payload, sizeof(char), 10, f_synthetic);
  }
  return (0);
}

